#!/usr/bin/perl -w

# Program cisco7.pl

# (c) Gary C. Kessler, 2012 [gck@garykessler.net]

# cisco7 -- Concept based on L0pht's cracker program for a Palm
#   Pilot that cracks Cisco passwords using the "type 7" hash.
# Based also on my C program, dated 3/3/2000

# ********************************************************
# MAIN PROGRAM BLOCK
# ********************************************************

# Initialize global variables

$version = "1.0";
$build_date = "24 April 2012";

$vector = "dsfd;kfoA,.iyewrkldJKDHSUBsgvca69834nc";
$vector_len = length ($vector);
$line = "";

print "\nCisco type 7 password cracker V$version - Gary C. Kessler ($build_date)\n\n";

# Parse command line switches.
# If user has supplied the -h, -v, or an illegal switch, the subroutine
# returns a value of 1 and the program stops.

if (parsecommandline ())
  { exit 1; };

# Transfer initial vector ($vector) into $vbyte array

for ($i=0; $i<$vector_len; $i++)
  { $vbyte [$i] = ord (substr ($vector,$i,1)); };

# Process the input ($line) and do some error checking. First, ensure that
# $line is an even number of characters.

$len = length ($line);
if ($len % 2 != 0)
  { c7error (1); }

# Now check for the presence of an invalid decimal number in the first two
# byte positions. If valid, calculate the offset and ensure that it is <16.

for ($i=0; $i<2; $i++)
      {
      $temp = uc (substr ($line,$i,1));
      if ($temp lt '0' || $temp gt '9')
        { c7error (6); }
      };

$offset = substr($line,0,1) * 10 + substr ($line,1,1);
if ($offset > 15)
  { c7error (2); }

# Now check for the presence of an invalid hex character in $line

for ($i=2; $i<$len; $i++)
  {
  $temp = uc (substr ($line,$i,1));
  if (not ($temp ge '0' && $temp le '9') &&
      not ($temp ge 'A' && $temp le 'F'))
    { c7error (5); }
  };

# Transfer $line into $lbyte array. While $len is the length of the
# $line password string, $j is the length of the string in bytes

$j = 0;
for ($i=0; $i<$len; $i+=2)
  { $lbyte [$j] = hex (substr ($line,$i,2)); $j++};

# More error checking! We have a problem if the length of the input is longer than
# the length of the vector *or* if the (length of the password plus offset) is
# longer than the vector. In either case, truncate the input, if necessary.

if (($j-1) > $vector_len)
  {
  c7error (3);
  $j = $j - ($offset + $j - $vector_len);
  }
if (($offset + $j -1) > $vector_len) 
  {
  c7error (4);
  $j = $j - ($offset + $j - $vector_len);
  }

# Whew! Well, if we've gotten this far, there are no errors in the input!
# The output is merely --

#          output [i] = input [i] XOR vector [i+offset]

for ($i=1; $i<$j; $i++)
  { $out_byte [$i] = $vbyte[$i+$offset-1] ^ $lbyte[$i]; };
print "Cleartext password is ", $j-1, " bytes in length (not including the brackets):\n[";
for ($i=1; $i<$j; $i++)
  { print chr ($out_byte [$i]); };
print "]\n\n";

print "Cheers!\n\n";


# ********************************************************
# *****           SUBROUTINE BLOCKS                  *****

# ********************************************************

# Print error messages and stop

sub c7error
{
my ($errnum) = @_;

CASE:
  {
  $errnum eq 1 && do
     {
     print "Fatal input error: Password string does not contain an even number of characters.\n\n";
     exit 1;
     };
  $errnum eq 2 && do
     {
     print "Fatal input error: Offset byte value ($offset) larger than 15.\n\n";
     exit 1;
     };
  $errnum eq 3 && do
     {
     print "Password length ($j) larger than vector ($vector_len bytes). Truncating input...\n\n";
     last CASE;
     };
  $errnum eq 4 && do
     {
     print "Offset ($offset) plus password length ($j) larger than vector ($vector_len bytes). Truncating input...\n\n";
     last CASE;
     };
  $errnum eq 5 && do
     {
     print "Fatal input error: Invalid hex character in password string.\n\n";
     exit 1;
     };
  $errnum eq 6 && do
     {
     print "Fatal input error: First two characters in password string must be numbers.\n\n";
     exit 1;
     };
  };
}

# ********************************************************
# help_text
# ********************************************************

# Display the help file

sub help_text
{
print<< "EOT";
Program usage: cisco7 -i string_to_decode
               cicso7 {[-h] | [-v]}

 where: -i is the Cisco type 7 password string to decode
        -h prints this help file
        -v displays the program version number

If the -i switch is missing, the program will ask for the string.

EOT

return;
}

# ********************************************************
# parsecommandline
# ********************************************************

# Parse command line for any switches that are present. Query
# user for any missing information

# Return $state = 1 to indicate that the program should stop
# immediately (-h, -v, or unrecognized switch)

sub parsecommandline
{
my $state = 0;

# Parse command line switches ($ARGV array of length $#ARGV)

if ($#ARGV >= 0)
  { 
  for ($i = 0; $i <= $#ARGV; $i++)
    {
    PARMS:
      {
      $ARGV[$i] eq "-i" && do
         {
         $line = $ARGV[$i+1];
         $i++;
         last PARMS;
         };
      $ARGV[$i] eq "-v" && do
         {
         $state = 1;
         return $state;
         };
      $ARGV[$i] eq "-h" && do
         {
         help_text();
         $state = 1;
         return $state;
         };
      do
         {
         print "Invalid parameter \"$ARGV[$i]\".\n\n";
         print "Usage: cisco7 {[-i string_to_decode] | [-h] | [-v]}\n\n";
         $state = 1;
         return $state;
         };
      };
    };
  }

# Prompt for input string, if not supplied

if ($line eq "")
  {
  print "Enter the string to decode: ";
  chomp ($line = <STDIN>);
  print "\n";
  };

return $state;
}

